// file      : examples/cxx/tree/wildcard/driver.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : not copyrighted - public domain

#include <string>
#include <memory>   // std::auto_ptr
#include <cstring>  // std::memcpy
#include <iostream>

#include <xercesc/dom/DOM.hpp>
#include <xercesc/util/PlatformUtils.hpp>

#include "email.hxx"

// The following string class keeps us sane when working with Xerces.
// Include it after the generated header in order to get only char or
// wchar_t version depending on how you compiled your schemas.
//
#include <xsd/cxx/xml/string.hxx>

using std::cerr;
using std::cout;
using std::endl;
using std::string;

int
main (int argc, char* argv[])
{
  if (argc != 2)
  {
    cerr << "usage: " << argv[0] << " email.xml" << endl;
    return 1;
  }

  using namespace xercesc;

  int r (0);

  // The Xerces-C++ DOM objects that will be used to store the
  // content matched by wildcards "out-live" the call to the
  // parsing function. Therefore we need to initialize the
  // Xerces-C++ runtime ourselves.
  //
  XMLPlatformUtils::Initialize ();

  try
  {
    using namespace s;
    namespace xml = xsd::cxx::xml;
    xml_schema::string text;

    // Read in the message.
    //
    std::auto_ptr<tRoot> root1 (
      elem (argv[1], xml_schema::flags::dont_initialize));

    // Print what we've got.
    //
    cerr << "Text:      " << root1->text() << endl;
    cerr << root1->withany().any().size() << endl;
    cerr << root1->withany().Private().size() << endl;

    tElement elem2;

/*
    DOMDocument& d = elem2.dom_document();
    DOMElement* domElem1 = d.createElement( XMLString::transcode("Private") );
    domElem1->setTextContent( XMLString::transcode( "222" ) );
    elem2.any().push_back( domElem1 );
*/
    tPrivate priv1;
    DOMDocument& d = priv1.dom_document();
    DOMElement* domElem1 ( d.createElement( XMLString::transcode("dontneedthis") ) );
    domElem1->setTextContent( XMLString::transcode( "222" ) );
    priv1.any().push_back( domElem1 );

    elem2.Private().push_back(priv1);

    tRoot root2( "aaa", elem2);

    xml_schema::namespace_infomap map;
    map["s"].name = "http://www.s/s";
    map["s"].schema = "email.xsd";

    elem ( std::cout,
             root2,
             map,
             "UTF-8",
             xml_schema::flags::dont_initialize );

  }
  catch (const xml_schema::exception& e)
  {
    cerr << e << endl;
    r = 1;
  }
  catch (const xercesc::DOMException& e) {
    cerr << XMLString::transcode(e.msg);
  }
  catch (...)
  {
    cerr << "unrecognized exception";
  }
  XMLPlatformUtils::Terminate ();
  return r;
}
